"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var updateCodeMirrorDocs_1 = __importDefault(require("./updateCodeMirrorDocs"));
var makeCodeMirrorChangeHandler_1 = __importDefault(require("./makeCodeMirrorChangeHandler"));
var Mutex_1 = __importDefault(require("./Mutex"));
/**
 * Connect an Automerge document
 *
 * @param watchableDoc - the Automerge document that will be connected to CodeMirror instances
 * @return ConnectCodeMirror - a function for connecting an Automerge.Text object in the document to a CodeMirror instance
 */
function connectAutomergeDoc(watchableDoc) {
    var doc = watchableDoc.get();
    var mutex = new Mutex_1.default();
    var codeMirrorMap = new Map();
    function getCodeMirror(textObjectId) {
        return codeMirrorMap.get(textObjectId);
    }
    var handler = function () {
        var newDoc = watchableDoc.get();
        updateCodeMirrorDocs_1.default(doc, newDoc, getCodeMirror, mutex);
        doc = newDoc;
    };
    function connectCodeMirror(codeMirror, getText) {
        codeMirror.setValue(getText(watchableDoc.get()).toString());
        if (codeMirrorMap.size === 0) {
            watchableDoc.registerHandler(handler);
        }
        var _a = makeCodeMirrorChangeHandler_1.default(watchableDoc, getText, mutex), textObjectId = _a.textObjectId, codeMirrorChangeHandler = _a.codeMirrorChangeHandler;
        codeMirror.on('change', codeMirrorChangeHandler);
        codeMirrorMap.set(textObjectId, codeMirror);
        function disconnectCodeMirror() {
            codeMirror.off('change', codeMirrorChangeHandler);
            codeMirrorMap.delete(textObjectId);
            if (codeMirrorMap.size === 0) {
                watchableDoc.unregisterHandler(handler);
            }
        }
        return disconnectCodeMirror;
    }
    return connectCodeMirror;
}
exports.default = connectAutomergeDoc;
//# sourceMappingURL=connectAutomergeDoc.js.map